package com.muyu.task.server.thread;

import java.util.concurrent.*;

public class OptimizedPrioritizedThreadPool {
    private static final ExecutorService executor;
    private static final Semaphore highPrioritySemaphore;
    private static final Semaphore mediumPrioritySemaphore;
    private static final Semaphore lowPrioritySemaphore;
    private static final Semaphore emergencySemaphore;

    private static final int totalThreads = 30; // 固定线程池大小
    private static final int defaultHighThreads = 16;
    private static final int defaultMediumThreads = 9;
    private static final int defaultLowThreads = 5;

    private static final int emergencyThreads = 14;
    private static final int emergencyHighThreads = 9;
    private static final int emergencyMediumThreads = 5;
    private static final int emergencyLowThreads = 2;

    private static volatile boolean inEmergencyMode = false;

    static {
        if (defaultHighThreads + defaultMediumThreads + defaultLowThreads > totalThreads) {
            throw new IllegalArgumentException("Default priority threads exceed total threads.");
        }
        if (emergencyThreads + emergencyHighThreads + emergencyMediumThreads + emergencyLowThreads > totalThreads) {
            throw new IllegalArgumentException("Emergency priority threads exceed total threads.");
        }

        // 创建固定大小的线程池
        executor = new ThreadPoolExecutor(
                totalThreads, totalThreads,
                60L, TimeUnit.SECONDS,
                new LinkedBlockingQueue<Runnable>() // 使用无界的阻塞队列
        );
        highPrioritySemaphore = new Semaphore(defaultHighThreads);
        mediumPrioritySemaphore = new Semaphore(defaultMediumThreads);
        lowPrioritySemaphore = new Semaphore(defaultLowThreads);
        emergencySemaphore = new Semaphore(0);
    }

    public static void submitEmergencyTask(Runnable task) {
        // 调整线程分配以适应紧急任务
        adjustForEmergency();

        try {
            emergencySemaphore.acquire();
            executor.submit(() -> {
                try {
                    task.run();
                } finally {
                    emergencySemaphore.release();
                    // 任务完成后恢复默认线程分配
                    restoreDefaults();
                }
            });
        } catch (InterruptedException e) {
            Thread.currentThread().interrupt();
            // 日志记录中断信息
            System.err.println("Interrupted while waiting to execute emergency task.");
        }
    }

    public static void submitHighPriorityTask(Runnable task) {
        try {
            highPrioritySemaphore.acquire();
            executor.submit(() -> {
                try {
                    task.run();
                } finally {
                    highPrioritySemaphore.release();
                }
            });
        } catch (InterruptedException e) {
            Thread.currentThread().interrupt();
            // 日志记录中断信息
            System.err.println("Interrupted while waiting to execute high priority task.");
        }
    }

    public static void submitMediumPriorityTask(Runnable task) {
        try {
            mediumPrioritySemaphore.acquire();
            executor.submit(() -> {
                try {
                    task.run();
                } finally {
                    mediumPrioritySemaphore.release();
                }
            });
        } catch (InterruptedException e) {
            Thread.currentThread().interrupt();
            // 日志记录中断信息
            System.err.println("Interrupted while waiting to execute medium priority task.");
        }
    }

    public static void submitLowPriorityTask(Runnable task) {
        try {
            lowPrioritySemaphore.acquire();
            executor.submit(() -> {
                try {
                    task.run();
                } finally {
                    lowPrioritySemaphore.release();
                }
            });
        } catch (InterruptedException e) {
            Thread.currentThread().interrupt();
            // 日志记录中断信息
            System.err.println("Interrupted while waiting to execute low priority task.");
        }
    }

    private static synchronized void adjustForEmergency() {
        // 仅在未处于紧急模式下调整线程分配
        if (!inEmergencyMode) {
            inEmergencyMode = true;

            // 释放一些高、中、低优先级的线程，以便为紧急任务腾出空间
            highPrioritySemaphore.release(defaultHighThreads - emergencyHighThreads);
            mediumPrioritySemaphore.release(defaultMediumThreads - emergencyMediumThreads);
            lowPrioritySemaphore.release(defaultLowThreads - emergencyLowThreads);

            // 为紧急任务分配线程
            emergencySemaphore.release(emergencyThreads);
        }
    }

    private static synchronized void restoreDefaults() {
        // 检查是否处于紧急模式
        if (inEmergencyMode) {
            // 清空紧急任务的许可
            emergencySemaphore.drainPermits();

            // 重新分配高、中、低优先级的线程
            try {
                highPrioritySemaphore.acquire(emergencyHighThreads);
                mediumPrioritySemaphore.acquire(emergencyMediumThreads);
                lowPrioritySemaphore.acquire(emergencyLowThreads);
            } catch (InterruptedException e) {
                throw new RuntimeException(e);
            }

            highPrioritySemaphore.release(defaultHighThreads);
            mediumPrioritySemaphore.release(defaultMediumThreads);
            lowPrioritySemaphore.release(defaultLowThreads);

            // 清空紧急任务的许可
            emergencySemaphore.release(-emergencyThreads);

            // 退出紧急模式
            inEmergencyMode = false;
        }
    }

    public static void shutdown() throws InterruptedException {
        executor.shutdown();
        if (!executor.awaitTermination(1, TimeUnit.HOURS)) {
            executor.shutdownNow();
            if (!executor.awaitTermination(1, TimeUnit.MINUTES)) {
                System.err.println("Pool did not terminate");
            }
        }
    }

}
